﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace WindowsForms1
{
    /**
     * <summary>
     * A desktop application to produce a life insurance quote for a customer.
     * </summary>
     */
    public partial class Form1 : Form
    {
        /**
         * <summary>
         * Calls the InitialComponent method in 'Form1.Designer.cs' which draws the Form.
         * </summary>
         */
        public Form1()
        {
            InitializeComponent();
        }

        /**
         * <summary>
         * Displays a startup message to the user in a dialog window
         * </summary>
         * 
         * <param name="sender">A reference to the Form object</param>
         * <param name="e">A reference to the Load event</param>
         */
        private void Form1_Load(object sender, EventArgs e)
        {
            MessageBox.Show("Please fill in the following 'Insurance Quote' form and submit", "Let's Get Started");
        }

        /**
         * <summary>
         * Calcuates an insurance quote from the 'Name' and 'Age' TextBoxes,
         * 'Female' and 'Male' RadioButtons, 'NonSmoker' CheckBox and
         * 'Region' ComboBox
         * </summary>
         * 
         * <param name="sender">A reference to the Button object</param>
         * <param name="e">A reference to the Click event</param>
         * 
         * <exception cref="Exception">
         * If 'Name' TextBox is blank or 
         * If neither 'Female' nor 'Male' RadioButton is selected 
         * </exception>
         * 
         * <exception cref="FormatException">
         * If 'Age' TextBox is non-numeric
         * </exception>
         */
        private void buttonGetQuote_Click(object sender, EventArgs e) 
        {
            try
            {
                const int BASE_RATE = 250;

                //int genderDiscount = 0, smokerDiscount = 0, regionCharge;

                if (textBoxName.Text == "")
                {
                    throw new Exception("Name is blank");
                }
                
                int ageRate = getAgeRate( Convert.ToInt32(textBoxAge.Text) );

                if (! radioButtonMale.Checked && ! radioButtonFemale.Checked )
                {
                    throw new Exception("Gender not selected");
                }

                int genderDiscount = 0;

                if (radioButtonFemale.Checked)
                {
                    genderDiscount = 50;
                }

                int smokerDiscount = 0;

                if (checkBoxNonSmoker.Checked)
                {
                    smokerDiscount = 75;
                }

                int regionCharge = getRegionCharge();

                int quote = getQuote(BASE_RATE, ageRate, genderDiscount, smokerDiscount, regionCharge);

                displayQuote(quote);
            }

            catch (FormatException ex)
            {
                MessageBox.Show("Invalid age: " + ex.Message, "Input Error");
            }

            catch (ArgumentException ex)
            {
                MessageBox.Show(ex.Message, "Input Error");
            }

            catch (Exception ex)
            {
                MessageBox.Show(ex.Message, "Input Error");
            }
        }

        /**
         * <summary>
         * Calculates and returns the portion of the insurance quote that is based on age.
         * If age is 20 or less return zero (0).
         * If age is greater than 20 return $5 times the number of years over 20
         * </summary>
         * 
         * <param name="age">The age of the person for the quote</param>
         * <returns>The portion of the insurance quote based on age</returns>
         * 
         * <exception cref="ArgumentException">
         * If the 'age' parameter is negative
         * </exception>
         */
        private int getAgeRate(int age)
        {
            if (age < 0)
            {
                throw new ArgumentException("Age may not be negative");
            }

            if (age <= 20)
            {
                return 0;
            }
            else
            {
                return (age - 20) * 5;
            }
        }

        /**
         * <summary>
         * Calculates and returns the portion of the insurance quote based upon 
         * the region of residence.
         * </summary>
         * 
         * <returns>
         * The portion of the insurance quote based the region of residence
         * </returns>
         * 
         * <exception cref="Exception">
         * If or no item is selected from the 'Region' ComboBox
         * </exception>
         */
        private int getRegionCharge()
        {
            if (comboBoxRegion.SelectedIndex == -1)
            {
                throw new Exception("Region of residence not selected");
            }

            if (comboBoxRegion.SelectedItem.ToString() == "East")
            {
                return 100;
            }
            else if (comboBoxRegion.SelectedItem.ToString() == "South")
            {
                return 65;
            }
            else if (comboBoxRegion.SelectedItem.ToString() == "Central")
            {
                return 75;
            }
            else // comboBoxRegion.SelectedItem.ToString() == "West"
            {
                return 95;
            }
        }

        /**
         * <summary>
         * Returns the insurance quote which is the base rate plus the age rate,
         * minus the gender discount and the smoker discount, 
         * plus the region of residence charge.
         * </summary>
         * 
         * <param name="baseRate">The base rate for an insurance quote</param>
         * <param name="ageRate">The amount added to the insurance quote for age over 20</param>
         * <param name="genderDiscount">The amount subtracted from the insurance quote for females</param>
         * <param name="smokerDiscount">The amount subtracted from the insurance quote for nonsmokers</param>
         * <param name="regionCharge">The amount added to the insurance quote depending upon the insured's region of residence</param>
         * 
         * <returns>The amount of the insurance quote</returns>
         */
        private int getQuote(int baseRate, int ageRate, int genderDiscount, int smokerDiscount, int regionCharge)
        {
            return baseRate + ageRate - genderDiscount - smokerDiscount + regionCharge;
        }

        /**
         * <summary>
         * Displays the client name, age, region of residence and the amount of 
         * the insurance quote to the 'Quote' ListBox.
         * </summary>
         * 
         * <param name="quote">The total amount of the insurance quote</param>
         */
        private void displayQuote(int quote)
        {
            listBoxQuote.Items.Clear();
            listBoxQuote.Items.Add("Name: " + textBoxName.Text);
            listBoxQuote.Items.Add("Age: " + textBoxAge.Text);
            listBoxQuote.Items.Add("Region: " + comboBoxRegion.SelectedItem.ToString() );
            listBoxQuote.Items.Add("Quote: $" + quote);
        }
    }
}
